package sun.security.krb5;

import sun.security.krb5.internal.Krb5;
import sun.security.util.SecurityProperties;
import sun.net.dns.ResolverConfiguration;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.security.PrivilegedAction;
import java.util.StringTokenizer;
import sun.security.action.GetPropertyAction;
import java.io.File;
import java.security.PrivilegedActionException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.security.AccessController;
import java.io.FileInputStream;
import java.security.PrivilegedExceptionAction;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;
import java.io.IOException;
import sun.security.krb5.internal.crypto.EType;
import java.util.Hashtable;

public class Config
{
    public static final boolean DISABLE_REFERRALS;
    public static final int MAX_REFERRALS;
    private static Config singleton;
    private Hashtable<String, Object> stanzaTable;
    private static boolean DEBUG;
    private static final int BASE16_0 = 1;
    private static final int BASE16_1 = 16;
    private static final int BASE16_2 = 256;
    private static final int BASE16_3 = 4096;
    private final String defaultRealm;
    private final String defaultKDC;
    
    private static native String getWindowsDirectory(boolean p0);
    
    public static synchronized Config getInstance() throws KrbException {
        if (Config.singleton == null) {
            Config.singleton = new Config();
        }
        return Config.singleton;
    }
    
    public static synchronized void refresh() throws KrbException {
        Config.singleton = new Config();
        KdcComm.initStatic();
        EType.initStatic();
        Checksum.initStatic();
        KrbAsReqBuilder.ReferralsState.initStatic();
    }
    
    private static boolean isMacosLionOrBetter() {
        if (!getProperty("os.name").contains("OS X")) {
            return false;
        }
        String[] split = getProperty("os.version").split("\\.");
        if (!split[0].equals("10")) {
            return false;
        }
        if (split.length < 2) {
            return false;
        }
        try {
            if (Integer.parseInt(split[1]) >= 7) {
                return true;
            }
        }
        catch (NumberFormatException ex) {}
        return false;
    }
    
    private Config() throws KrbException {
        this.stanzaTable = new Hashtable();
        String property = getProperty("java.security.krb5.kdc");
        if (property != null) {
            this.defaultKDC = property.replace(':', ' ');
        }
        else {
            this.defaultKDC = null;
        }
        this.defaultRealm = getProperty("java.security.krb5.realm");
        if ((this.defaultKDC == null && this.defaultRealm != null) || (this.defaultRealm == null && this.defaultKDC != null)) {
            throw new KrbException("System property java.security.krb5.kdc and java.security.krb5.realm both must be set or neither must be set.");
        }
        try {
            String javaFileName = this.getJavaFileName();
            if (javaFileName != null) {
                this.stanzaTable = this.parseStanzaTable(this.loadConfigFile(javaFileName));
                if (Config.DEBUG) {
                    System.out.println("Loaded from Java config");
                }
            }
            else {
                boolean n = false;
                if (isMacosLionOrBetter()) {
                    try {
                        this.stanzaTable = SCDynamicStoreConfig.getConfig();
                        if (Config.DEBUG) {
                            System.out.println("Loaded from SCDynamicStoreConfig");
                        }
                        n = true;
                    }
                    catch (IOException ex) {}
                }
                if (n == 0) {
                    this.stanzaTable = this.parseStanzaTable(this.loadConfigFile(this.getNativeFileName()));
                    if (Config.DEBUG) {
                        System.out.println("Loaded from native config");
                    }
                }
            }
        }
        catch (IOException ex2) {}
        return;
    }
    
    public String get(String... array) {
        Vector<String> string0 = this.getString0(array);
        if (string0 == null) {
            return null;
        }
        return (String)string0.lastElement();
    }
    
    public Boolean getBooleanObject(String... array) {
        String value = this.get(array);
        if (value == null) {
            return null;
        }
        String lowerCase = value.toLowerCase(Locale.US);
        int n = -1;
        switch (lowerCase.hashCode()) {
            case 119527: {
                if (lowerCase.equals("yes")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 3569038: {
                if (lowerCase.equals("true")) {
                    n = 1;
                    break;
                }
                break;
            }
            case 3521: {
                if (lowerCase.equals("no")) {
                    n = 2;
                    break;
                }
                break;
            }
            case 97196323: {
                if (lowerCase.equals("false")) {
                    n = 3;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0:
            case 1: {
                return Boolean.TRUE;
            }
            case 2:
            case 3: {
                return Boolean.FALSE;
            }
            default: {
                return null;
            }
        }
    }
    
    public String getAll(String... array) {
        Vector<String> string0 = this.getString0(array);
        if (string0 == null) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        int n = 1;
        for (String s : string0) {
            if (n != 0) {
                sb.append(s);
                n = 0;
            }
            else {
                sb.append(' ').append(s);
            }
        }
        return sb.toString();
    }
    
    public boolean exists(String... array) {
        return this.get0(array) != null;
    }
    
    private Vector<String> getString0(String... array) {
        try {
            return (Vector)this.get0(array);
        }
        catch (ClassCastException cause) {
            throw new IllegalArgumentException(cause);
        }
    }
    
    private Object get0(String... array) {
        Hashtable<Object, Object> hashtable = this.stanzaTable;
        try {
            int length = array.length;
            int i = 0;
            while (i < length) {
                hashtable = ((Hashtable)hashtable).get(array[i]);
                if (hashtable == null) {
                    return null;
                }
                ++i;
            }
            return hashtable;
        }
        catch (ClassCastException cause) {
            throw new IllegalArgumentException(cause);
        }
    }
    
    public int getIntValue(String... array) {
        String value = this.get(array);
        int intValue = Integer.MIN_VALUE;
        if (value != null) {
            try {
                intValue = this.parseIntValue(value);
            }
            catch (NumberFormatException ex) {
                if (Config.DEBUG) {
                    System.out.println("Exception in getting value of " + Arrays.toString(array) + " " + ex.getMessage());
                    System.out.println("Setting " + Arrays.toString(array) + " to minimum value");
                }
                intValue = Integer.MIN_VALUE;
            }
        }
        return intValue;
    }
    
    public boolean getBooleanValue(String... array) {
        String value = this.get(array);
        return value != null && value.equalsIgnoreCase("true");
    }
    
    private int parseIntValue(String s) throws NumberFormatException {
        int int1 = 0;
        if (s.startsWith("+")) {
            return Integer.parseInt(s.substring(1));
        }
        if (s.startsWith("0x")) {
            char[] charArray = s.substring(2).toCharArray();
            if (charArray.length > 8) {
                throw new NumberFormatException();
            }
            for (int i = 0; i < charArray.length; ++i) {
                int n = charArray.length - i - 1;
                switch (charArray[i]) {
                    case '0': {
                        int1 += 0;
                        break;
                    }
                    case '1': {
                        int1 += 1 * this.getBase(n);
                        break;
                    }
                    case '2': {
                        int1 += 2 * this.getBase(n);
                        break;
                    }
                    case '3': {
                        int1 += 3 * this.getBase(n);
                        break;
                    }
                    case '4': {
                        int1 += 4 * this.getBase(n);
                        break;
                    }
                    case '5': {
                        int1 += 5 * this.getBase(n);
                        break;
                    }
                    case '6': {
                        int1 += 6 * this.getBase(n);
                        break;
                    }
                    case '7': {
                        int1 += 7 * this.getBase(n);
                        break;
                    }
                    case '8': {
                        int1 += 8 * this.getBase(n);
                        break;
                    }
                    case '9': {
                        int1 += 9 * this.getBase(n);
                        break;
                    }
                    case 'A':
                    case 'a': {
                        int1 += 10 * this.getBase(n);
                        break;
                    }
                    case 'B':
                    case 'b': {
                        int1 += 11 * this.getBase(n);
                        break;
                    }
                    case 'C':
                    case 'c': {
                        int1 += 12 * this.getBase(n);
                        break;
                    }
                    case 'D':
                    case 'd': {
                        int1 += 13 * this.getBase(n);
                        break;
                    }
                    case 'E':
                    case 'e': {
                        int1 += 14 * this.getBase(n);
                        break;
                    }
                    case 'F':
                    case 'f': {
                        int1 += 15 * this.getBase(n);
                        break;
                    }
                    default: {
                        throw new NumberFormatException("Invalid numerical format");
                    }
                }
            }
            if (int1 < 0) {
                throw new NumberFormatException("Data overflow.");
            }
        }
        else {
            int1 = Integer.parseInt(s);
        }
        return int1;
    }
    
    private int getBase(int n) {
        int n2 = 16;
        switch (n) {
            case 0: {
                n2 = 1;
                break;
            }
            case 1: {
                n2 = 16;
                break;
            }
            case 2: {
                n2 = 256;
                break;
            }
            case 3: {
                n2 = 4096;
                break;
            }
            default: {
                for (int i = 1; i < n; ++i) {
                    n2 *= 16;
                }
                break;
            }
        }
        return n2;
    }
    
    private List<String> loadConfigFile(final String s) throws IOException, KrbException {
        try {
            ArrayList<String> list = new ArrayList();
            try (final BufferedReader bufferedReader = new BufferedReader(new InputStreamReader((InputStream)AccessController.doPrivileged(new PrivilegedExceptionAction<FileInputStream>() {
                @Override
                public FileInputStream run() throws IOException {
                    return new FileInputStream(s);
                }
            })))) {
                String str = null;
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    String trim = line.trim();
                    if (trim.isEmpty() || trim.startsWith("#")) {
                        continue;
                    }
                    if (trim.startsWith(";")) {
                        continue;
                    }
                    if (trim.startsWith("[")) {
                        if (!trim.endsWith("]")) {
                            throw new KrbException("Illegal config content:" + trim);
                        }
                        if (str != null) {
                            list.add(str);
                            list.add("}");
                        }
                        String trim2 = trim.substring(1, trim.length() - 1).trim();
                        if (trim2.isEmpty()) {
                            throw new KrbException("Illegal config content:" + trim);
                        }
                        str = trim2 + " = {";
                    }
                    else if (trim.startsWith("{")) {
                        if (str == null) {
                            throw new KrbException("Config file should not start with \"{\"");
                        }
                        str += " {";
                        if (trim.length() <= 1) {
                            continue;
                        }
                        list.add(str);
                        str = trim.substring(1).trim();
                    }
                    else {
                        if (str == null) {
                            continue;
                        }
                        list.add(str);
                        str = trim;
                    }
                }
                if (str != null) {
                    list.add(str);
                    list.add("}");
                }
            }
            return list;
        }
        catch (PrivilegedActionException ex) {
            throw (IOException)ex.getException();
        }
    }
    
    private Hashtable<String, Object> parseStanzaTable(List<String> list) throws KrbException {
        Hashtable<String, Object> stanzaTable = this.stanzaTable;
        for (String str : list) {
            if (str.equals("}")) {
                stanzaTable = (Object)stanzaTable.remove(" PARENT ");
                if (stanzaTable == null) {
                    throw new KrbException("Unmatched close brace");
                }
                continue;
            }
            else {
                int index = str.indexOf(61);
                if (index < 0) {
                    throw new KrbException("Illegal config content:" + str);
                }
                String s = str.substring(0, index).trim();
                String trimmed = trimmed(str.substring(index + 1));
                if (trimmed.equals("{")) {
                    if (stanzaTable == this.stanzaTable) {
                        s = s.toLowerCase(Locale.US);
                    }
                    Hashtable value2 = new Hashtable();
                    stanzaTable.put(s, value2);
                    value2.put(" PARENT ", stanzaTable);
                    stanzaTable = value2;
                }
                else {
                    Vector<String> value;
                    if (stanzaTable.containsKey(s)) {
                        if (!(stanzaTable.get(s) instanceof Vector)) {
                            throw new KrbException("Key " + s + "used for both value and section");
                        }
                        value = (Object)stanzaTable.get(s);
                    }
                    else {
                        value = new Vector();
                        stanzaTable.put(s, value);
                    }
                    value.add(trimmed);
                }
            }
        }
        if (stanzaTable != this.stanzaTable) {
            throw new KrbException("Not closed");
        }
        return stanzaTable;
    }
    
    private String getJavaFileName() {
        String str = getProperty("java.security.krb5.conf");
        if (str == null) {
            str = getProperty("java.home") + File.separator + "lib" + File.separator + "security" + File.separator + "krb5.conf";
            if (!this.fileExists(str)) {
                str = null;
            }
        }
        if (Config.DEBUG) {
            System.out.println("Java config name: " + str);
            return str;
        }
        return str;
    }
    
    private String getNativeFileName() {
        String macosConfigFile = null;
        String property = getProperty("os.name");
        if (property.startsWith("Windows")) {
            try {
                Credentials.ensureLoaded();
            }
            catch (Exception ex) {}
            if (Credentials.alreadyLoaded) {
                String windowsDirectory = getWindowsDirectory(false);
                if (windowsDirectory != null) {
                    String s;
                    if (windowsDirectory.endsWith("\\")) {
                        s = windowsDirectory + "krb5.ini";
                    }
                    else {
                        s = windowsDirectory + "\\krb5.ini";
                    }
                    if (this.fileExists(s)) {
                        macosConfigFile = s;
                    }
                }
                if (macosConfigFile == null) {
                    String windowsDirectory2 = getWindowsDirectory(true);
                    if (windowsDirectory2 != null) {
                        String s2;
                        if (windowsDirectory2.endsWith("\\")) {
                            s2 = windowsDirectory2 + "krb5.ini";
                        }
                        else {
                            s2 = windowsDirectory2 + "\\krb5.ini";
                        }
                        macosConfigFile = s2;
                    }
                }
            }
            if (macosConfigFile == null) {
                macosConfigFile = "c:\\winnt\\krb5.ini";
            }
        }
        else if (property.startsWith("SunOS")) {
            macosConfigFile = "/etc/krb5/krb5.conf";
        }
        else if (property.contains("OS X")) {
            macosConfigFile = this.findMacosConfigFile();
        }
        else {
            macosConfigFile = "/etc/krb5.conf";
        }
        if (Config.DEBUG) {
            System.out.println("Native config name: " + macosConfigFile);
            return macosConfigFile;
        }
        return macosConfigFile;
    }
    
    private static String getProperty(String theProp) {
        return (String)AccessController.doPrivileged(new GetPropertyAction(theProp));
    }
    
    private String findMacosConfigFile() {
        String string = getProperty("user.home") + "/Library/Preferences/edu.mit.Kerberos";
        if (this.fileExists(string)) {
            return string;
        }
        if (this.fileExists("/Library/Preferences/edu.mit.Kerberos")) {
            return "/Library/Preferences/edu.mit.Kerberos";
        }
        return "/etc/krb5.conf";
    }
    
    private static String trimmed(String s) {
        s = s.trim();
        if (s.length() >= 2 && ((s.charAt(0) == '\"' && s.charAt(s.length() - 1) == '\"') || (s.charAt(0) == '\'' && s.charAt(s.length() - 1) == '\''))) {
            s = s.substring(1, s.length() - 1).trim();
            return s;
        }
        return s;
    }
    
    public void listTable() {
        System.out.println(this);
    }
    
    public int[] defaultEtype(String str) throws KrbException {
        String value = this.get(new String[] { "libdefaults", str });
        int[] builtInDefaults;
        if (value == null) {
            if (Config.DEBUG) {
                System.out.println("Using builtin default etypes for " + str);
            }
            builtInDefaults = EType.getBuiltInDefaults();
        }
        else {
            String delim = " ";
            int i = 0;
            while (i < value.length()) {
                if (value.substring(i, i + 1).equals(",")) {
                    delim = ",";
                    break;
                }
                ++i;
            }
            StringTokenizer o = new StringTokenizer(value, delim);
            int countTokens = o.countTokens();
            ArrayList o2 = new ArrayList(countTokens);
            for (int j = 0; j < countTokens; ++j) {
                int type = getType(o.nextToken());
                if (type != -1 && EType.isSupported(type)) {
                    o2.add(Integer.valueOf(type));
                }
            }
            if (o2.isEmpty()) {
                throw new KrbException("no supported default etypes for " + str);
            }
            builtInDefaults = new int[o2.size()];
            for (int k = 0; k < builtInDefaults.length; ++k) {
                builtInDefaults[k] = (int)o2.get(k);
            }
        }
        if (Config.DEBUG) {
            System.out.print("default etypes for " + str + ":");
            for (int l = 0; l < builtInDefaults.length; ++l) {
                System.out.print(" " + builtInDefaults[l]);
            }
            System.out.println(".");
            return builtInDefaults;
        }
        return builtInDefaults;
    }
    
    public static int getType(String s) {
        int n = -1;
        if (s == null) {
            return n;
        }
        if (s.startsWith("d") || s.startsWith("D")) {
            if (s.equalsIgnoreCase("des-cbc-crc")) {
                n = 1;
            }
            else if (s.equalsIgnoreCase("des-cbc-md5")) {
                n = 3;
            }
            else if (s.equalsIgnoreCase("des-mac")) {
                n = 4;
            }
            else if (s.equalsIgnoreCase("des-mac-k")) {
                n = 5;
            }
            else if (s.equalsIgnoreCase("des-cbc-md4")) {
                n = 2;
            }
            else if (s.equalsIgnoreCase("des3-cbc-sha1") || s.equalsIgnoreCase("des3-hmac-sha1") || s.equalsIgnoreCase("des3-cbc-sha1-kd") || s.equalsIgnoreCase("des3-cbc-hmac-sha1-kd")) {
                n = 16;
            }
        }
        else if (s.startsWith("a") || s.startsWith("A")) {
            if (s.equalsIgnoreCase("aes128-cts") || s.equalsIgnoreCase("aes128-cts-hmac-sha1-96")) {
                n = 17;
            }
            else if (s.equalsIgnoreCase("aes256-cts") || s.equalsIgnoreCase("aes256-cts-hmac-sha1-96")) {
                n = 18;
            }
            else if (s.equalsIgnoreCase("arcfour-hmac") || s.equalsIgnoreCase("arcfour-hmac-md5")) {
                n = 23;
            }
        }
        else if (s.equalsIgnoreCase("rc4-hmac")) {
            n = 23;
        }
        else if (s.equalsIgnoreCase("CRC32")) {
            n = 1;
        }
        else if (s.startsWith("r") || s.startsWith("R")) {
            if (s.equalsIgnoreCase("rsa-md5")) {
                n = 7;
            }
            else if (s.equalsIgnoreCase("rsa-md5-des")) {
                n = 8;
            }
        }
        else if (s.equalsIgnoreCase("hmac-sha1-des3-kd")) {
            n = 12;
        }
        else if (s.equalsIgnoreCase("hmac-sha1-96-aes128")) {
            n = 15;
        }
        else if (s.equalsIgnoreCase("hmac-sha1-96-aes256")) {
            n = 16;
        }
        else if (s.equalsIgnoreCase("hmac-md5-rc4") || s.equalsIgnoreCase("hmac-md5-arcfour") || s.equalsIgnoreCase("hmac-md5-enc")) {
            n = -138;
        }
        else if (s.equalsIgnoreCase("NULL")) {
            n = 0;
        }
        return n;
    }
    
    public void resetDefaultRealm(String str) {
        if (Config.DEBUG) {
            System.out.println(">>> Config try resetting default kdc " + str);
        }
    }
    
    public boolean useAddresses() {
        String value = this.get(new String[] { "libdefaults", "no_addresses" });
        int n = value != null && value.equalsIgnoreCase("false");
        if (n == 0) {
            String value2 = this.get(new String[] { "libdefaults", "noaddresses" });
            n = (value2 != null && value2.equalsIgnoreCase("false"));
        }
        return n;
    }
    
    private boolean useDNS(String s, boolean b) {
        Boolean booleanObject = this.getBooleanObject(new String[] { "libdefaults", s });
        if (booleanObject != null) {
            return (boolean)booleanObject;
        }
        Boolean booleanObject2 = this.getBooleanObject(new String[] { "libdefaults", "dns_fallback" });
        if (booleanObject2 != null) {
            return (boolean)booleanObject2;
        }
        return b;
    }
    
    private boolean useDNS_KDC() {
        return this.useDNS("dns_lookup_kdc", true);
    }
    
    private boolean useDNS_Realm() {
        return this.useDNS("dns_lookup_realm", false);
    }
    
    public String getDefaultRealm() throws KrbException {
        if (this.defaultRealm != null) {
            return this.defaultRealm;
        }
        KrbException cause = null;
        String s = this.get(new String[] { "libdefaults", "default_realm" });
        if (s == null && this.useDNS_Realm()) {
            try {
                s = this.getRealmFromDNS();
            }
            catch (KrbException ex) {
                cause = ex;
            }
        }
        if (s == null) {
            s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                @Override
                public String run() {
                    if (System.getProperty("os.name").startsWith("Windows")) {
                        return System.getenv("USERDNSDOMAIN");
                    }
                    return null;
                }
            });
        }
        if (s == null) {
            KrbException o = new KrbException("Cannot locate default realm");
            if (cause != null) {
                o.initCause(cause);
            }
            throw o;
        }
        else {
            return s;
        }
    }
    
    public String getKDCList(String defaultRealm) throws KrbException {
        if (defaultRealm == null) {
            defaultRealm = this.getDefaultRealm();
        }
        if (defaultRealm.equalsIgnoreCase(this.defaultRealm)) {
            return this.defaultKDC;
        }
        KrbException cause = null;
        String s = this.getAll(new String[] { "realms", defaultRealm, "kdc" });
        if (s == null && this.useDNS_KDC()) {
            try {
                s = this.getKDCFromDNS(defaultRealm);
            }
            catch (KrbException ex) {
                cause = ex;
            }
        }
        if (s == null) {
            s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                @Override
                public String run() {
                    if (System.getProperty("os.name").startsWith("Windows")) {
                        String s = System.getenv("LOGONSERVER");
                        if (s != null && s.startsWith("\\\\")) {
                            s = s.substring(2);
                            return s;
                        }
                        return s;
                    }
                    else {
                        return null;
                    }
                }
            });
        }
        if (s == null) {
            if (this.defaultKDC != null) {
                return this.defaultKDC;
            }
            KrbException o = new KrbException("Cannot locate KDC");
            if (cause != null) {
                o.initCause(cause);
            }
            throw o;
        }
        else {
            return s;
        }
    }
    
    private String getRealmFromDNS() throws KrbException {
        String s = null;
        String canonicalHostName;
        try {
            canonicalHostName = InetAddress.getLocalHost().getCanonicalHostName();
        }
        catch (UnknownHostException cause) {
            KrbException ex = new KrbException(60, "Unable to locate Kerberos realm: " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        String mapHostToRealm = PrincipalName.mapHostToRealm(canonicalHostName);
        if (mapHostToRealm == null) {
            Iterator<String> iterator = ResolverConfiguration.open().searchlist().iterator();
            while (iterator.hasNext()) {
                s = checkRealm((String)iterator.next());
                if (s != null) {
                    break;
                }
                continue;
            }
        }
        else {
            s = checkRealm(mapHostToRealm);
        }
        if (s == null) {
            throw new KrbException(60, "Unable to locate Kerberos realm");
        }
        return s;
    }
    
    private static String checkRealm(String realmComponent) {
        if (Config.DEBUG) {
            System.out.println("getRealmFromDNS: trying " + realmComponent);
        }
        String[] kerberosService;
        for (kerberosService = null, ; kerberosService == null && realmComponent != null; kerberosService = KrbServiceLocator.getKerberosService(realmComponent), realmComponent = Realm.parseRealmComponent(realmComponent)) {}
        if (kerberosService != null) {
            int i = 0;
            while (i < kerberosService.length) {
                if (kerberosService[i].equalsIgnoreCase(realmComponent)) {
                    return kerberosService[i];
                }
                ++i;
            }
        }
        return null;
    }
    
    private String getKDCFromDNS(String str) throws KrbException {
        String string = "";
        if (Config.DEBUG) {
            System.out.println("getKDCFromDNS using UDP");
        }
        String[] array = KrbServiceLocator.getKerberosService(str, "_udp");
        if (array == null) {
            if (Config.DEBUG) {
                System.out.println("getKDCFromDNS using TCP");
            }
            array = KrbServiceLocator.getKerberosService(str, "_tcp");
        }
        if (array == null) {
            throw new KrbException(60, "Unable to locate KDC for realm " + str);
        }
        if (array.length == 0) {
            return null;
        }
        for (int i = 0; i < array.length; ++i) {
            string = string + array[i].trim() + " ";
        }
        String trim = string.trim();
        if (trim.equals("")) {
            return null;
        }
        return trim;
    }
    
    private boolean fileExists(String s) {
        return (boolean)AccessController.doPrivileged(new FileExistsAction(s));
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        toStringInternal("", this.stanzaTable, sb);
        return sb.toString();
    }
    
    private static void toStringInternal(String str, Object obj, StringBuffer sb) {
        if ((obj instanceof String)) {
            sb.append(obj).append('\n');
        }
        else if ((obj instanceof Hashtable)) {
            Hashtable<Object, Object> hashtable = (Hashtable)obj;
            sb.append("{\n");
            for (Object next : hashtable.keySet()) {
                sb.append(str).append("    ").append(next).append(" = ");
                toStringInternal(str + "    ", hashtable.get(next), sb);
            }
            sb.append(str).append("}\n");
        }
        else if ((obj instanceof Vector)) {
            Vector<Object> vector = (Vector)obj;
            sb.append("[");
            int n = 1;
            Object[] array = vector.toArray();
            for (int length = array.length, i = 0; i < length; n = 0, ++i) {
                Object obj2 = array[i];
                if (n == 0) {
                    sb.append(",");
                }
                sb.append(obj2);
            }
            sb.append("]\n");
        }
    }
    
    static {
        String privilegedGetOverridable = SecurityProperties.privilegedGetOverridable("sun.security.krb5.disableReferrals");
        if (privilegedGetOverridable != null) {
            DISABLE_REFERRALS = "true".equalsIgnoreCase(privilegedGetOverridable);
        }
        else {
            DISABLE_REFERRALS = false;
        }
        int int1 = 5;
        String privilegedGetOverridable2 = SecurityProperties.privilegedGetOverridable("sun.security.krb5.maxReferrals");
        try {
            int1 = Integer.parseInt(privilegedGetOverridable2);
        }
        catch (NumberFormatException ex) {}
        MAX_REFERRALS = int1;
        Config.singleton = null;
        Config.DEBUG = Krb5.DEBUG;
    }
    
    static class FileExistsAction implements PrivilegedAction<Boolean>
    {
        private String fileName;
        
        public FileExistsAction(String fileName) {
            this.fileName = fileName;
        }
        
        @Override
        public Boolean run() {
            return Boolean.valueOf(new File(this.fileName).exists());
        }
    }
}
